@@ -4,8 +4,9 @@ author: Maikel Punie <maikel.punie@gmail.com>
 from __future__ import annotations
 
 import asyncio
+import math
 import string
-from typing import Any, Callable
+from typing import TYPE_CHECKING, Any, Awaitable, Callable
 
 from velbusaio.command_registry import commandRegistry
 from velbusaio.const import (
@@ -16,6 +17,11 @@ from velbusaio.const import (
     VOLUME_CUBIC_METER_HOUR,
     VOLUME_LITERS_HOUR,
 )
+from velbusaio.message import Message
+from velbusaio.messages.module_status import PROGRAM_SELECTION
+
+if TYPE_CHECKING:
+    from velbusaio.module import Module
 
 
 class Channel:
@@ -24,7 +30,15 @@ class Channel:
     This is the basic abstract class of a velbus channel
     """
 
-    def __init__(self, module, num, name, nameEditable, writer, address):
+    def __init__(
+        self,
+        module: Module,
+        num: int,
+        name: str,
+        nameEditable: bool,
+        writer: Callable[[Message], Awaitable[None]],
+        address: int,
+    ):
         self._num = num
         self._module = module
         self._name = name
@@ -37,7 +51,7 @@ class Channel:
         self._on_status_update = []
         self._name_parts = {}
 
-    def get_module_type(self) -> str:
+    def get_module_type(self) -> int:
         return self._module.get_type()
 
     def get_module_type_name(self) -> str:
@@ -46,7 +60,7 @@ class Channel:
     def get_module_serial(self) -> str:
         return self._module.get_serial()
 
-    def get_module_address(self, chan_type="") -> int:
+    def get_module_address(self, chan_type: str = "") -> int:
         """Return (sub)module address for channel"""
         if chan_type == "Button" and self._num > 24:
             return self._module.get_addresses()[3]
@@ -69,8 +83,6 @@ class Channel:
     def is_loaded(self) -> bool:
         """
         Is this channel loaded
-
-        :return: Boolean
         """
         return self._is_loaded
 
@@ -86,16 +98,16 @@ class Channel:
         """
         return self._name
 
-    def set_name_char(self, pos, char) -> None:
+    def set_name_char(self, pos: int, char: int) -> None:
         self._is_loaded = True
         self._name_parts = {}
         # make sure the string is long enough
         while len(self._name) < int(pos):
             self._name += " "
         # store the char on correct pos
-        self._name = self._name[: int(pos)] + str(char) + self._name[int(pos) + 1 :]
+        self._name = self._name[: int(pos)] + chr(char) + self._name[int(pos) + 1 :]
 
-    def set_name_part(self, part, name) -> None:
+    def set_name_part(self, part: int, name: str) -> None:
         """
         Set a part of the channel name
         """
@@ -127,14 +139,14 @@ class Channel:
         self._on_status_update = []
         self._name_parts = {}
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         items = []
         for k, v in self.__dict__.items():
             if k not in ["_module", "_writer", "_name_parts"]:
                 items.append(f"{k} = {v!r}")
         return "{}[{}]".format(type(self), ", ".join(items))
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.__repr__()
 
     def get_channel_info(self) -> dict[str, Any]:
@@ -156,30 +168,30 @@ class Channel:
                 for m in self._on_status_update:
                     await m()
 
-    def get_categories(self) -> list:
+    def get_categories(self) -> list[str]:
         """
         Get the categories (mainly for home-assistant)
         """
         # COMPONENT_TYPES = ["switch", "sensor", "binary_sensor", "cover", "climate", "light"]
         return []
 
-    def on_status_update(self, meth: Callable) -> None:
+    def on_status_update(self, meth: Callable[[], Awaitable[None]]) -> None:
         self._on_status_update.append(meth)
 
-    def get_counter_state(self):
-        raise NotImplementedError
+    def get_counter_state(self) -> int:
+        raise NotImplementedError()
 
-    def get_counter_unit(self):
-        raise NotImplementedError
+    def get_counter_unit(self) -> str:
+        raise NotImplementedError()
 
     def get_max(self) -> int:
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def get_min(self) -> int:
-        raise NotImplementedError
+        raise NotImplementedError()
 
     async def press(self) -> None:
-        raise NotImplementedError
+        raise NotImplementedError()
 
 
 class Blind(Channel):
@@ -190,9 +202,9 @@ class Blind(Channel):
     _state = None
     # State reports the direction of *movement*: moving up, moving down or stopped
     _position = None
-    # Position reporting is not supported by VBMxBL modules (only in BLE/BLS)
+    # Position reporting is not supported by VMBxBL modules (only in BLE/BLS)
 
-    def get_categories(self) -> list:
+    def get_categories(self) -> list[str]:
         return ["cover"]
 
     def get_position(self) -> int | None:
@@ -225,7 +237,7 @@ class Blind(Channel):
 
     def support_position(self) -> bool:
         # position will be populated after the first BlindStatusNgMessage (during module load)
-        # For VBMxBL modules, position will remain None and not be overwritten
+        # For VMBxBL modules, position will remain None and not be overwritten
         return self._position is not None
 
     async def open(self) -> None:
@@ -264,7 +276,7 @@ class Button(Channel):
     _closed = False
     _led_state = None
 
-    def get_categories(self) -> list:
+    def get_categories(self) -> list[str]:
         if self._enabled:
             return ["binary_sensor", "led", "button"]
         return []
@@ -286,8 +298,6 @@ class Button(Channel):
     async def set_led_state(self, state: str) -> None:
         """
         Set led
-
-        :return: None
         """
         if state == "on":
             code = 0xF6
@@ -311,8 +321,6 @@ class Button(Channel):
     async def press(self) -> None:
         """
         Press the button
-
-        :return: None
         """
         _mod_add = self.get_module_address("Button")
         _chn_num = self._num - self._module.calc_channel_offset(_mod_add)
@@ -340,7 +348,7 @@ class ButtonCounter(Button):
     _counter = None
     _delay = None
 
-    def get_categories(self) -> list:
+    def get_categories(self) -> list[str]:
         if self._counter:
             return ["sensor"]
         return ["binary_sensor", "button"]
@@ -368,7 +376,7 @@ class ButtonCounter(Button):
             val = 0
         return round(val, 2)
 
-    def get_unit(self) -> str:
+    def get_unit(self) -> str | None:
         if self._Unit in (
             VOLUME_LITERS_HOUR,
             VOLUME_CUBIC_METER_HOUR,
@@ -391,7 +399,7 @@ class Sensor(Button):
     A sensor in this case is actually a Button
     """
 
-    def get_categories(self) -> list:
+    def get_categories(self) -> list[str]:
         if self._enabled:
             return ["binary_sensor", "led"]
         return []
@@ -412,7 +420,14 @@ class Dimmer(Channel):
     _state: int = 0
 
     def __init__(
-        self, module, num, name, nameEditable, writer, address, slider_scale: int = 100
+        self,
+        module: Module,
+        num: int,
+        name: str,
+        nameEditable: bool,
+        writer: Callable[[Message], Awaitable[None]],
+        address: int,
+        slider_scale: int = 100,
     ):
         super().__init__(module, num, name, nameEditable, writer, address)
 
@@ -420,7 +435,7 @@ class Dimmer(Channel):
         # VMB4DC has dim values 0(off), 1-99(dimmed), 100(full on)
         # VMBDALI has dim values 0(off), 1-253(dimmed), 254(full on), 255(previous value)
 
-    def get_categories(self) -> list:
+    def get_categories(self) -> list[str]:
         return ["light"]
 
     def is_on(self) -> bool:
@@ -437,7 +452,7 @@ class Dimmer(Channel):
         """
         return int(self._state * 100 / self.slider_scale)
 
-    async def set_dimmer_state(self, slider, transitiontime=0) -> None:
+    async def set_dimmer_state(self, slider: int, transitiontime: int = 0) -> None:
         """
         Set dimmer to slider
         """
@@ -448,7 +463,7 @@ class Dimmer(Channel):
         msg.dimmer_channels = [self._num]
         await self._writer(msg)
 
-    async def restore_dimmer_state(self, transitiontime=0) -> None:
+    async def restore_dimmer_state(self, transitiontime: int = 0) -> None:
         """
         restore dimmer to last known state
         """
@@ -465,14 +480,16 @@ class Temperature(Channel):
     """
 
     _cur = 0
+    _cur_precision = None
     _max = None
     _min = None
     _target = 0
     _cmode = None
     _cstatus = None
     _thermostat = False
+    _sleep_timer = 0
 
-    def get_categories(self) -> list:
+    def get_categories(self) -> list[str]:
         if self._thermostat:
             return ["sensor", "climate"]
         return ["sensor"]
@@ -489,12 +506,12 @@ class Temperature(Channel):
     def is_temperature(self) -> bool:
         return True
 
-    def get_max(self) -> int:
+    def get_max(self) -> int | None:
         if self._max is None:
             return None
         return round(self._max, 2)
 
-    def get_min(self) -> int:
+    def get_min(self) -> int | None:
         if self._min is None:
             return None
         return round(self._min, 2)
@@ -508,34 +525,87 @@ class Temperature(Channel):
     def get_climate_mode(self) -> str:
         return self._cstatus
 
-    async def set_temp(self, temp) -> None:
+    async def set_temp(self, temp: float) -> None:
         cls = commandRegistry.get_command(0xE4, self._module.get_type())
         msg = cls(self._address)
-        msg.temp = temp * 2
+        msg.temp = temp * 2  # TODO: int()
         await self._writer(msg)
 
-    async def set_preset(self, mode) -> None:
-        if mode == "safe":
+    async def _switch_mode(self) -> None:
+        if self._cmode == "safe":
             code = 0xDE
-        elif mode == "comfort":
+        elif self._cmode == "comfort":
             code = 0xDB
-        elif mode == "day":
+        elif self._cmode == "day":
             code = 0xDC
-        elif mode == "night":
+        else:  # "night"
             code = 0xDD
+
+        if self._cstatus == "run":
+            sleep = 0x0
+        elif self._cstatus == "manual":
+            sleep = 0xFFFF
+        elif self._cstatus == "sleep":
+            sleep = self._sleep_timer
+        else:
+            sleep = 0x0
         cls = commandRegistry.get_command(code, self._module.get_type())
-        msg = cls(self._address)
+        msg = cls(self._address, sleep)
         await self._writer(msg)
 
-    async def set_mode(self, mode) -> None:
+    async def set_preset(self, preset: str) -> None:
+        self._cmode = preset
+        await self._switch_mode()
+
+    async def set_climate_mode(self, mode: str) -> None:
+        self._cstatus = mode
+        await self._switch_mode()
+
+    async def set_mode(self, mode: str) -> None:
+        # TODO: change function name, proposal = set_heat_cool_mode
         if mode == "heat":
             code = 0xE0
         elif mode == "cool":
             code = 0xDF
+        # TODO: else case
         cls = commandRegistry.get_command(code, self._module.get_type())
         msg = cls(self._address)
         await self._writer(msg)
 
+    async def maybe_update_temperature(self, new_temp: float, precision: float) -> None:
+        # Based on experiments, Velbus modules seem to truncate (i.e. round down)
+        current_temp_rounded_to_precision = (
+            math.floor(self._cur / precision) * precision
+        )
+
+        if current_temp_rounded_to_precision == new_temp:
+            # The newly received temperature is still in line with our current value,
+            # but with reduced precision.
+            # Don't update (would lose high precision)
+            return
+
+        elif (
+            current_temp_rounded_to_precision - precision
+            <= new_temp
+            < current_temp_rounded_to_precision
+            and self._cur_precision < precision
+        ):
+            # The newly received temperature is 1 LSb below the current value
+            # and the current value was set by a better precision message
+            # Modify the received temperature by "adding precision", while still keeping the same low precision value
+            # e.g. (decimal digits represent precision)
+            # | Actual  | Msg     | Stored  |
+            # | 21.0000 | 21.0000 | 21.0000 |
+            # | 20.9375 | 20.5    | 20.9375 |
+            new_temp = current_temp_rounded_to_precision - self._cur_precision
+
+        await self.update(
+            {
+                "cur": new_temp,
+                "cur_precision": precision,
+            }
+        )
+
 
 class SensorNumber(Channel):
     """
@@ -544,16 +614,16 @@ class SensorNumber(Channel):
 
     _cur = 0
 
-    def get_categories(self):
+    def get_categories(self) -> list[str]:
         return ["sensor"]
 
-    def get_class(self):
+    def get_class(self) -> None:
         return None
 
-    def get_unit(self):
+    def get_unit(self) -> None:
         return None
 
-    def get_state(self):
+    def get_state(self) -> float:
         return round(self._cur, 2)
 
 
@@ -564,16 +634,16 @@ class LightSensor(Channel):
 
     _cur = 0
 
-    def get_categories(self):
+    def get_categories(self) -> list[str]:
         return ["sensor"]
 
-    def get_class(self):
+    def get_class(self) -> str:
         return DEVICE_CLASS_ILLUMINANCE
 
-    def get_unit(self):
+    def get_unit(self) -> None:
         return None
 
-    def get_state(self):
+    def get_state(self) -> float:
         return round(self._cur, 2)
 
 
@@ -588,7 +658,7 @@ class Relay(Channel):
     _forced_on = False
     _disabled = False
 
-    def get_categories(self) -> list:
+    def get_categories(self) -> list[str]:
         if self._enabled:
             return ["switch"]
         return []
@@ -653,3 +723,32 @@ class Memo(Channel):
                 msg = cls(self._address)
                 msg.start = msgcntr
         await self._writer(msg)
+
+
+class SelectedProgram(Channel):
+    """
+    A selected program channel
+    """
+
+    _selected_program_str = None
+
+    def get_categories(self) -> list[str]:
+        return ["select"]
+
+    def get_class(self) -> None:
+        return None
+
+    def get_options(self) -> list:
+        return PROGRAM_SELECTION.values()
+
+    def get_selected_program(self) -> str:
+        return self._selected_program_str
+
+    async def set_selected_program(self, program_str: str) -> None:
+        self._selected_program_str = program_str
+        command_code = 0xB3
+        cls = commandRegistry.get_command(command_code, self._module.get_type())
+        index = list(PROGRAM_SELECTION.values()).index(program_str)
+        program = list(PROGRAM_SELECTION.keys())[index]
+        msg = cls(self._address, program)
+        await self._writer(msg)
