@@ -8,6 +8,7 @@ import asyncio
 import json
 import logging
 import re
+from typing import TYPE_CHECKING, Awaitable, Callable
 
 import pkg_resources
 
@@ -18,13 +19,20 @@ from velbusaio.messages.module_subtype import ModuleSubTypeMessage
 from velbusaio.messages.module_type import ModuleTypeMessage
 from velbusaio.raw_message import RawMessage
 
+if TYPE_CHECKING:
+    from velbusaio.controller import Velbus
+
 
 class PacketHandler:
     """
     The packetHandler class
     """
 
-    def __init__(self, writer, velbus) -> None:
+    def __init__(
+        self,
+        writer: Callable[[Message], Awaitable[None]],
+        velbus: Velbus,
+    ) -> None:
         self._log = logging.getLogger("velbus-packet")
         self._writer = writer
         self._velbus = velbus
