@@ -1,12 +1,15 @@
 """
 :author: Thomas Delaet <thomas@delaet.org>
 """
-from velbusaio.command_registry import register_command
+from __future__ import annotations
+
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xD9
 
 
+@register(COMMAND_CODE)
 class BusErrorStatusRequestMessage(Message):
     """
     send by:
@@ -27,6 +30,3 @@ class BusErrorStatusRequestMessage(Message):
         :return: bytes
         """
         return bytes([COMMAND_CODE])
-
-
-register_command(COMMAND_CODE, BusErrorStatusRequestMessage)
