@@ -3,14 +3,13 @@
 """
 from __future__ import annotations
 
-import json
-
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xBE
 
 
+@register(COMMAND_CODE)
 class KwhStatusMessage(Message):
     """
     send by: VMB7IN
@@ -51,6 +50,3 @@ class KwhStatusMessage(Message):
         :return: list
         """
         return self.channel
-
-
-register_command(COMMAND_CODE, KwhStatusMessage)
