@@ -3,12 +3,13 @@
 """
 from __future__ import annotations
 
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xFA
 
 
+@register(COMMAND_CODE)
 class ModuleStatusRequestMessage(Message):
     """
     send by:
@@ -36,6 +37,3 @@ class ModuleStatusRequestMessage(Message):
         :return: bytes
         """
         return bytes([COMMAND_CODE, self.channels_to_byte(self.channels)])
-
-
-register_command(COMMAND_CODE, ModuleStatusRequestMessage)
