@@ -5,7 +5,7 @@ from __future__ import annotations
 
 import struct
 
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xFF
@@ -25,6 +25,7 @@ MODULES_WITHOUT_SERIAL = {
 }
 
 
+@register(COMMAND_CODE)
 class ModuleTypeMessage(Message):
     """
     send by: VMB6IN, VMB4RYLD
@@ -80,6 +81,3 @@ class ModuleTypeMessage(Message):
                 self.build_week,
             ]
         )
-
-
-register_command(COMMAND_CODE, ModuleTypeMessage)
