@@ -3,12 +3,13 @@
 """
 from __future__ import annotations
 
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xC9
 
 
+@register(COMMAND_CODE)
 class ReadDataBlockFromMemoryMessage(Message):
     """
     send by:
@@ -37,6 +38,3 @@ class ReadDataBlockFromMemoryMessage(Message):
         :return: bytes
         """
         return bytes([COMMAND_CODE, self.high_address, self.low_address])
-
-
-register_command(COMMAND_CODE, ReadDataBlockFromMemoryMessage)
