@@ -3,14 +3,13 @@
 """
 from __future__ import annotations
 
-import json
-
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xE6
 
 
+@register(COMMAND_CODE)
 class SensorTemperatureMessage(Message):
     """
     send by: VMBTS, vmbg*pd, ...
@@ -55,6 +54,3 @@ class SensorTemperatureMessage(Message):
             self.max = -127 + (((data[4] << 8) | data[5]) / 32 * 0.0625)
         else:
             self.max = (((data[4] << 8) | data[5]) / 32) * 0.0625
-
-
-register_command(COMMAND_CODE, SensorTemperatureMessage)
