@@ -3,14 +3,13 @@
 """
 from __future__ import annotations
 
-import json
-
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xF6
 
 
+@register(COMMAND_CODE)
 class SetLedMessage(Message):
     """
     send by: VMB4RYLD
@@ -37,6 +36,3 @@ class SetLedMessage(Message):
         :return: bytes
         """
         return bytes([COMMAND_CODE, self.channels_to_byte(self.leds)])
-
-
-register_command(COMMAND_CODE, SetLedMessage)
