@@ -3,15 +3,15 @@
 """
 from __future__ import annotations
 
-import json
 import time
 
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xD8
 
 
+@register(COMMAND_CODE)
 class SetRealtimeClock(Message):
     """
     received by all modules
@@ -51,6 +51,3 @@ class SetRealtimeClock(Message):
         :return: bytes
         """
         return bytes([COMMAND_CODE, self._wday, self._hour, self._min])
-
-
-register_command(COMMAND_CODE, SetRealtimeClock)
