@@ -1,12 +1,15 @@
 """
 :author: Maikel Punie <maikel.punie@gmail.com>
 """
-from velbusaio.command_registry import register_command
+from __future__ import annotations
+
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xE4
 
 
+@register(COMMAND_CODE)
 class SetTemperatureMessage(Message):
     """
     send by: VMB4RYLD
@@ -36,6 +39,3 @@ class SetTemperatureMessage(Message):
         :return: bytes
         """
         return bytes([COMMAND_CODE, int(self.temp_type), int(self.temp)])
-
-
-register_command(COMMAND_CODE, SetTemperatureMessage)
