@@ -1,14 +1,17 @@
 """
 :author: Thomas Delaet <thomas@delaet.org>
 """
+from __future__ import annotations
+
 import struct
 
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0x03
 
 
+@register(COMMAND_CODE)
 class StartRelayTimerMessage(Message):
     """
     send by:
@@ -46,6 +49,3 @@ class StartRelayTimerMessage(Message):
             bytes([COMMAND_CODE, self.channels_to_byte(self.relay_channels)])
             + struct.pack(">L", self.delay_time)[-3:]
         )
-
-
-register_command(COMMAND_CODE, StartRelayTimerMessage)
