@@ -3,14 +3,13 @@
 """
 from __future__ import annotations
 
-import json
-
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xDE
 
 
+@register(COMMAND_CODE)
 class SwitchToSafeMessage(Message):
     """
     send by:
@@ -33,7 +32,4 @@ class SwitchToSafeMessage(Message):
         """
         :return: bytes
         """
-        return bytes([COMMAND_CODE, 0x00, 0x00])
-
-
-register_command(COMMAND_CODE, SwitchToSafeMessage)
+        return bytes([COMMAND_CODE, self.sleep >> 8, self.sleep & 0xFF])
