@@ -3,14 +3,13 @@
 """
 from __future__ import annotations
 
-import json
-
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xDF
 
 
+@register(COMMAND_CODE)
 class TempSetCoolingMessage(Message):
     """
     send by:
@@ -28,18 +27,8 @@ class TempSetCoolingMessage(Message):
         self.needs_no_rtr(rtr)
         self.set_attributes(priority, address, rtr)
 
-    def to_json(self):
-        """
-        :return: str
-        """
-        json_dict = self.to_json_basic()
-        return json.dumps(json_dict)
-
     def data_to_binary(self):
         """
         :return: bytes
         """
         return bytes([COMMAND_CODE, 0xAA])
-
-
-register_command(COMMAND_CODE, TempSetCoolingMessage)
