@@ -3,14 +3,13 @@
 """
 from __future__ import annotations
 
-import json
-
-from velbusaio.command_registry import register_command
+from velbusaio.command_registry import register
 from velbusaio.message import Message
 
 COMMAND_CODE = 0xE0
 
 
+@register(COMMAND_CODE)
 class TempSetHeatingMessage(Message):
     """
     send by:
@@ -33,6 +32,3 @@ class TempSetHeatingMessage(Message):
         :return: bytes
         """
         return bytes([COMMAND_CODE, 0xAA])
-
-
-register_command(COMMAND_CODE, TempSetHeatingMessage)
