@@ -9,26 +9,34 @@ import pathlib
 import pickle
 import struct
 import sys
-from typing import Optional
+from typing import Awaitable, Callable
 
 from velbusaio.channels import (
     Blind,
     Button,
     ButtonCounter,
+    Channel,
     Dimmer,
     EdgeLit,
     LightSensor,
     Memo,
     Relay,
+    SelectedProgram,
     Sensor,
     SensorNumber,
     Temperature,
     ThermostatChannel,
 )
 from velbusaio.command_registry import commandRegistry
-from velbusaio.const import CHANNEL_LIGHT_VALUE, CHANNEL_MEMO_TEXT, PRIORITY_LOW
+from velbusaio.const import (
+    CHANNEL_LIGHT_VALUE,
+    CHANNEL_MEMO_TEXT,
+    CHANNEL_SELECTED_PROGRAM,
+    PRIORITY_LOW,
+)
 from velbusaio.helpers import handle_match, keys_exists
 from velbusaio.message import Message
+from velbusaio.messages import DaliDeviceSettingMsg
 from velbusaio.messages.blind_status import BlindStatusMessage, BlindStatusNgMessage
 from velbusaio.messages.channel_name_part1 import (
     ChannelNamePart1Message,
@@ -52,12 +60,21 @@ from velbusaio.messages.channel_name_request import ChannelNameRequestMessage
 from velbusaio.messages.clear_led import ClearLedMessage
 from velbusaio.messages.counter_status import CounterStatusMessage
 from velbusaio.messages.counter_status_request import CounterStatusRequestMessage
+from velbusaio.messages.dali_device_settings import DeviceType as DaliDeviceType
+from velbusaio.messages.dali_device_settings import DeviceTypeMsg as DaliDeviceTypeMsg
+from velbusaio.messages.dali_device_settings import MemberOfGroupMsg
+from velbusaio.messages.dali_device_settings_request import (
+    COMMAND_CODE as DALI_DEVICE_SETTINGS_REQUEST_COMMAND_CODE,
+)
+from velbusaio.messages.dali_device_settings_request import DaliDeviceSettingsRequest
+from velbusaio.messages.dali_dim_value_status import DimValueStatus
 from velbusaio.messages.dimmer_channel_status import DimmerChannelStatusMessage
 from velbusaio.messages.dimmer_status import DimmerStatusMessage
 from velbusaio.messages.fast_blinking_led import FastBlinkingLedMessage
 from velbusaio.messages.memory_data import MemoryDataMessage
 from velbusaio.messages.meteo_raw import MeteoRawMessage
 from velbusaio.messages.module_status import (
+    ModuleStatusGP4PirMessage,
     ModuleStatusMessage,
     ModuleStatusMessage2,
     ModuleStatusPirMessage,
@@ -81,12 +98,47 @@ class Module:
     Abstract class for Velbus hardware modules.
     """
 
+    @classmethod
+    def factory(
+        cls,
+        module_address: int,
+        module_type: int,
+        module_data: dict,
+        serial: int | None = None,
+        memorymap: int | None = None,
+        build_year: int | None = None,
+        build_week: int | None = None,
+        cache_dir: str | None = None,
+    ) -> Module:
+        if module_type == 0x45:
+            return VmbDali(
+                module_address,
+                module_type,
+                module_data,
+                serial,
+                memorymap,
+                build_year,
+                build_week,
+                cache_dir,
+            )
+
+        return Module(
+            module_address,
+            module_type,
+            module_data,
+            serial,
+            memorymap,
+            build_year,
+            build_week,
+            cache_dir,
+        )
+
     def __init__(
         self,
         module_address: int,
         module_type: int,
         module_data: dict,
-        serial: str = "",
+        serial: int | None = None,
         memorymap: int | None = None,
         build_year: int | None = None,
         build_week: int | None = None,
@@ -98,7 +150,7 @@ class Module:
 
         self._name = {}
         self._sub_address = {}
-        self.serial: str = serial
+        self.serial = serial
         self.memory_map_version = memorymap
         self.build_year = build_year
         self.build_week = build_week
@@ -107,13 +159,22 @@ class Module:
         self._channels = {}
         self.loaded = False
 
-    def initialize(self, writer: type) -> None:
+    def initialize(self, writer: Callable[[Message], Awaitable[None]]) -> None:
         self._log = logging.getLogger("velbus-module")
         self._writer = writer
         for chan in self._channels.values():
             chan._writer = writer
 
     def cleanupSubChannels(self) -> None:
+        # TODO: 21/11/2022 DannyDeGaspari: Fix needed
+        # Care should be taken for this function, not all subaddresses have their channels on multiples of 8.
+        # The last subaddress contain typically the temperature channels, has more then 8 channels
+        # and doesn't start on a boundary of 8.
+        # E.g. The VMBGP4 has one subaddress, so since the second subaddress is not defined,
+        # this function will delete channels 17-24 while 17 and 18 belong to the temperature channels.
+        #
+        # The solution would be that this functions knows were the temperature channels are located
+        # and/or what the max number of subaddresses are for each module.
         if self._sub_address == {}:
             assert "No subaddresses defined"
         for sub in range(1, 4):
@@ -228,35 +289,72 @@ class Module:
         elif isinstance(message, MemoryDataMessage):
             await self._process_memory_data_message(message)
         elif isinstance(message, (RelayStatusMessage, RelayStatusMessage2)):
-            await self._channels[message.channel].update(
+            await self._update_channel(
+                message.channel,
                 {
                     "on": message.is_on(),
                     "inhibit": message.is_inhibited(),
                     "forced_on": message.is_forced_on(),
                     "disabled": message.is_disabled(),
-                }
+                },
             )
         elif isinstance(message, SensorTemperatureMessage):
             chan = self._translate_channel_name(self._data["TemperatureChannel"])
-            await self._channels[chan].update(
+            await self._channels[chan].maybe_update_temperature(
+                message.getCurTemp(), 1 / 64
+            )
+            await self._update_channel(
+                chan,
                 {
-                    "cur": message.getCurTemp(),
                     "min": message.getMinTemp(),
                     "max": message.getMaxTemp(),
-                }
+                },
             )
         elif isinstance(message, TempSensorStatusMessage):
             # update the current temp
             chan = self._translate_channel_name(self._data["TemperatureChannel"])
             if chan in self._channels:
-                await self._channels[chan].update(
+                await self._update_channel(
+                    chan,
                     {
-                        "cur": message.current_temp,
                         "target": message.target_temp,
                         "cmode": message.mode_str,
                         "cstatus": message.status_str,
-                    }
+                        "sleep_timer": message.sleep_timer,
+                    },
                 )
+                await self._channels[chan].maybe_update_temperature(
+                    message.current_temp, 1 / 2
+                )
+            # update the thermostat channels
+            channel_name_to_msg_prop_map = {
+                "Heater": "heater",
+                "Boost": "boost",
+                "Pump": "pump",
+                "Cooler": "cooler",
+                "Alarm 1": "alarm1",
+                "Alarm 2": "alarm2",
+                "Alarm 3": "alarm3",
+                "Alarm 4": "alarm4",
+            }
+            for channel_str in self._data["Channels"]:
+                if keys_exists(self._data, "Channels", channel_str, "Type"):
+                    if (
+                        self._data["Channels"][channel_str]["Type"]
+                        == "ThermostatChannel"
+                    ):
+                        channel = self._translate_channel_name(channel_str)
+                        channel_name = self._data["Channels"][channel_str]["Name"]
+                        if channel in self._channels:
+                            await self._update_channel(
+                                channel,
+                                {
+                                    "closed": getattr(
+                                        message,
+                                        channel_name_to_msg_prop_map[channel_name],
+                                    )
+                                },
+                            )
         elif isinstance(message, PushButtonStatusMessage):
             _update_buttons = False
             for channel_types in self._data["Channels"]:
@@ -273,112 +371,150 @@ class Module:
                 for channel_id in range(1, 9):
                     channel = self._translate_channel_name(channel_id + _channel_offset)
                     if channel_id in message.closed:
-                        await self._channels[channel].update({"closed": True})
+                        await self._update_channel(channel, {"closed": True})
                     if channel_id in message.opened:
-                        await self._channels[channel].update({"closed": False})
+                        await self._update_channel(channel, {"closed": False})
         elif isinstance(message, (ModuleStatusMessage)):
             for channel_id in range(1, 9):
                 channel = self._translate_channel_name(channel_id + _channel_offset)
                 if channel_id in message.closed:
-                    await self._channels[channel].update({"closed": True})
+                    await self._update_channel(channel, {"closed": True})
                 elif channel in self._channels and isinstance(
                     self._channels[channel], (Button, ButtonCounter)
                 ):
-                    await self._channels[channel].update({"closed": False})
+                    await self._update_channel(channel, {"closed": False})
         elif isinstance(message, (ModuleStatusMessage2)):
             for channel_id in range(1, 9):
                 channel = self._translate_channel_name(channel_id + _channel_offset)
                 if channel_id in message.closed:
-                    await self._channels[channel].update({"closed": True})
+                    await self._update_channel(channel, {"closed": True})
                 elif isinstance(self._channels[channel], (Button, ButtonCounter)):
-                    await self._channels[channel].update({"closed": False})
+                    await self._update_channel(channel, {"closed": False})
                 if channel_id in message.enabled:
-                    await self._channels[channel].update({"enabled": True})
+                    await self._update_channel(channel, {"enabled": True})
                 elif channel in self._channels and isinstance(
                     self._channels[channel], (Button, ButtonCounter)
                 ):
-                    await self._channels[channel].update({"enabled": False})
+                    await self._update_channel(channel, {"enabled": False})
+            # self.selected_program_str = message.selected_program_str
+            await self._update_channel(
+                CHANNEL_SELECTED_PROGRAM,
+                {"selected_program_str": message.selected_program_str},
+            )
         elif isinstance(message, CounterStatusMessage) and isinstance(
             self._channels[message.channel], ButtonCounter
         ):
             channel = self._translate_channel_name(message.channel)
-            await self._channels[channel].update(
+            await self._update_channel(
+                channel,
                 {
                     "pulses": message.pulses,
                     "counter": message.counter,
                     "delay": message.delay,
-                }
+                },
             )
         elif isinstance(message, ModuleStatusPirMessage):
-            await self._channels[CHANNEL_LIGHT_VALUE].update(
-                {"cur": message.light_value}
+            await self._update_channel(
+                CHANNEL_LIGHT_VALUE, {"cur": message.light_value}
             )
-            await self._channels[1].update({"closed": message.dark})
-            await self._channels[2].update({"closed": message.light})
-            await self._channels[3].update({"closed": message.motion1})
-            await self._channels[4].update({"closed": message.light_motion1})
-            await self._channels[5].update({"closed": message.motion2})
-            await self._channels[6].update({"closed": message.light_motion2})
+            await self._update_channel(1, {"closed": message.dark})
+            await self._update_channel(2, {"closed": message.light})
+            await self._update_channel(3, {"closed": message.motion1})
+            await self._update_channel(4, {"closed": message.light_motion1})
+            await self._update_channel(5, {"closed": message.motion2})
+            await self._update_channel(6, {"closed": message.light_motion2})
             if 7 in self._channels:
-                await self._channels[7].update({"closed": message.low_temp_alarm})
+                await self._update_channel(7, {"closed": message.low_temp_alarm})
             if 8 in self._channels:
-                await self._channels[8].update({"closed": message.high_temp_alarm})
+                await self._update_channel(8, {"closed": message.high_temp_alarm})
+            # self.selected_program_str = message.selected_program_str
+            await self._update_channel(
+                CHANNEL_SELECTED_PROGRAM,
+                {"selected_program_str": message.selected_program_str},
+            )
+        elif isinstance(message, ModuleStatusGP4PirMessage):
+            await self._update_channel(
+                CHANNEL_LIGHT_VALUE, {"cur": message.light_value}
+            )
+            for channel_id in range(1, 9):
+                channel = self._translate_channel_name(channel_id + _channel_offset)
+                await self._update_channel(
+                    channel, {"closed": channel_id in message.closed}
+                )
+                if type(self._channels[channel]) == Button:
+                    # only treat 'enabled' if the channel is a Button
+                    await self._update_channel(
+                        channel, {"enabled": channel_id in message.enabled}
+                    )
+            # self.selected_program_str = message.selected_program_str
+            await self._update_channel(
+                CHANNEL_SELECTED_PROGRAM,
+                {"selected_program_str": message.selected_program_str},
+            )
         elif isinstance(message, UpdateLedStatusMessage):
             for channel_id in range(1, 9):
                 channel = self._translate_channel_name(channel_id + _channel_offset)
                 if channel_id in message.led_slow_blinking:
-                    await self._channels[channel].update({"led_state": "slow"})
+                    await self._update_channel(channel, {"led_state": "slow"})
                 if channel_id in message.led_fast_blinking:
-                    await self._channels[channel].update({"led_state": "fast"})
+                    await self._update_channel(channel, {"led_state": "fast"})
                 if channel_id in message.led_on:
-                    await self._channels[channel].update({"led_state": "on"})
+                    await self._update_channel(channel, {"led_state": "on"})
                 if (
                     channel_id not in message.led_slow_blinking
                     and channel_id not in message.led_fast_blinking
                     and channel_id not in message.led_on
                 ):
-                    await self._channels[channel].update({"led_state": "off"})
+                    await self._update_channel(channel, {"led_state": "off"})
         elif isinstance(message, SetLedMessage):
             for channel_id in range(1, 9):
                 channel = self._translate_channel_name(channel_id + _channel_offset)
                 if channel_id in message.leds:
-                    await self._channels[channel].update({"led_state": "on"})
+                    await self._update_channel(channel, {"led_state": "on"})
         elif isinstance(message, ClearLedMessage):
             for channel_id in range(1, 9):
                 channel = self._translate_channel_name(channel_id + _channel_offset)
                 if channel_id in message.leds:
-                    await self._channels[channel].update({"led_state": "off"})
+                    await self._update_channel(channel, {"led_state": "off"})
         elif isinstance(message, SlowBlinkingLedMessage):
             for channel_id in range(1, 9):
                 channel = self._translate_channel_name(channel_id + _channel_offset)
                 if channel_id in message.leds:
-                    await self._channels[channel].update({"led_state": "slow"})
+                    await self._update_channel(channel, {"led_state": "slow"})
         elif isinstance(message, FastBlinkingLedMessage):
             for channel_id in range(1, 9):
                 channel = self._translate_channel_name(channel_id + _channel_offset)
                 if channel_id in message.leds:
-                    await self._channels[channel].update({"led_state": "fast"})
+                    await self._update_channel(channel, {"led_state": "fast"})
         elif isinstance(message, (DimmerChannelStatusMessage, DimmerStatusMessage)):
             channel = self._translate_channel_name(message.channel)
-            await self._channels[channel].update({"state": message.cur_dimmer_state()})
+            await self._update_channel(channel, {"state": message.cur_dimmer_state()})
         elif isinstance(message, SliderStatusMessage):
             channel = self._translate_channel_name(message.channel)
-            await self._channels[channel].update({"state": message.cur_slider_state()})
+            await self._update_channel(channel, {"state": message.cur_slider_state()})
         elif isinstance(message, BlindStatusNgMessage):
             channel = self._translate_channel_name(message.channel)
-            await self._channels[channel].update(
-                {"state": message.status, "position": message.position}
+            await self._update_channel(
+                channel, {"state": message.status, "position": message.position}
             )
         elif isinstance(message, BlindStatusMessage):
             channel = self._translate_channel_name(message.channel)
-            await self._channels[channel].update({"state": message.status})
+            await self._update_channel(channel, {"state": message.status})
         elif isinstance(message, MeteoRawMessage):
-            await self._channels[11].update({"cur": message.rain})
-            await self._channels[12].update({"cur": message.light})
-            await self._channels[13].update({"cur": message.wind})
+            await self._update_channel(11, {"cur": message.rain})
+            await self._update_channel(12, {"cur": message.light})
+            await self._update_channel(23, {"cur": message.wind})
 
         self._cache()
 
+    async def _update_channel(self, channel: int, updates: dict):
+        try:
+            await self._channels[channel].update(updates)
+        except KeyError:
+            self._log.error(
+                f"channel {channel} does not exist for module @ address {self}"
+            )
+
     def get_channels(self) -> dict:
         """
         List all channels for this module
@@ -442,16 +578,17 @@ class Module:
         if "ModuleName" in mdata and isinstance(self._name, dict):
             # if self._name is a dict we are still loading
             # if its a string it was already complete
-            if message.data == 0xFF:
-                # modulename is complete
-                self._name = "".join(str(x) for x in self._name.values())
-            else:
-                char = mdata["ModuleName"].split(":")[0]
-                self._name[int(char)] = chr(message.data)
+            char_and_save = mdata["ModuleName"].split(":")
+            char = char_and_save[0]
+            self._name[int(char)] = chr(message.data)
+            if len(char_and_save) > 1 and char_and_save[1] == "Save":
+                self._name = "".join(
+                    str(x) for x in self._name.values() if x != chr(0xFF)
+                )
         elif "Match" in mdata:
             for chan, chan_data in handle_match(mdata["Match"], message.data).items():
                 data = chan_data.copy()
-                await self._channels[chan].update(data)
+                await self._update_channel(chan, data)
         elif "SensorName" in mdata:
             # this is part of the channel names
             # make sure we set the channel to loaded
@@ -465,7 +602,7 @@ class Module:
             chan = self._translate_channel_name(chan)
             self._channels[chan].set_name_char(pos, message.data)
         else:
-            print(mdata)
+            self._log.debug(mdata)
 
     def _process_channel_name_message(self, part: int, message: Message) -> None:
         channel = self._translate_channel_name(message.channel)
@@ -581,4 +718,148 @@ class Module:
                 if "Thermostat" in self._data or (
                     "ThermostatAddr" in self._data and self._data["ThermostatAddr"] != 0
                 ):
-                    await self._channels[int(chan)].update({"thermostat": True})
+                    await self._update_channel(int(chan), {"thermostat": True})
+        # add extra channel for program selection which is not in the channel list of the protocol.json file,
+        # but is available in the messages list of the corresponding module.
+        if keys_exists(self._data, "Messages", "B3"):
+            self._channels[CHANNEL_SELECTED_PROGRAM] = SelectedProgram(
+                self,
+                CHANNEL_SELECTED_PROGRAM,
+                "Selected Program",
+                False,
+                self._writer,
+                self._address,
+            )
+
+
+class VmbDali(Module):
+    """
+    DALI has a variable number of channels: the number of channels
+    depends on the number of DALI devices on the DALI bus
+    """
+
+    def __init__(
+        self,
+        module_address: int,
+        module_type: int,
+        module_data: dict,
+        serial: int | None = None,
+        memorymap: int | None = None,
+        build_year: int | None = None,
+        build_week: int | None = None,
+        cache_dir: str | None = None,
+    ) -> None:
+        super().__init__(
+            module_address,
+            module_type,
+            module_data,
+            serial,
+            memorymap,
+            build_year,
+            build_week,
+            cache_dir,
+        )
+        self.group_members: dict[int, set[int]] = {}
+
+    async def load(self, from_cache: bool = False) -> None:
+        await super().load(from_cache)
+
+        if not from_cache:
+            for chan in range(1, 64 + 1):
+                self._channels[chan] = Channel(
+                    self, chan, "placeholder", True, self._writer, self._address
+                )
+                # Placeholders will keep this module loading
+                # Until the DaliDeviceSettings messages either delete or replace these placeholder's
+                # with actual channels
+
+            await self._request_dali_channels()
+
+    async def _request_dali_channels(self):
+        msg_type = commandRegistry.get_command(
+            DALI_DEVICE_SETTINGS_REQUEST_COMMAND_CODE, self.get_type()
+        )
+        msg: DaliDeviceSettingsRequest = msg_type(self._address)
+        msg.priority = PRIORITY_LOW
+        msg.channel = 81  # all
+        msg.settings = None  # all
+        await self._writer(msg)
+
+    async def on_message(self, message: Message) -> None:
+        if isinstance(message, DaliDeviceSettingMsg):
+            if isinstance(message.data, DaliDeviceTypeMsg):
+                if message.data.device_type == DaliDeviceType.NoDevicePresent:
+                    if message.channel in self._channels:
+                        del self._channels[message.channel]
+                elif message.data.device_type == DaliDeviceType.LedModule:
+                    if self._channels.get(message.channel).__class__ != Dimmer:
+                        # New or changed type, replace channel:
+                        self._channels[message.channel] = Dimmer(
+                            self,
+                            message.channel,
+                            None,
+                            True,
+                            self._writer,
+                            self._address,
+                            slider_scale=254,
+                        )
+                        await self._request_single_channel_name(message.channel)
+
+            elif isinstance(message.data, MemberOfGroupMsg):
+                for group in range(0, 15 + 1):
+                    this_group_members = self.group_members.setdefault(group, set())
+                    if message.data.member_of_group[group]:
+                        this_group_members.add(message.channel)
+                    elif message.channel in this_group_members:
+                        this_group_members.remove(message.channel)
+
+        elif isinstance(message, PushButtonStatusMessage):
+            _channel_offset = self.calc_channel_offset(message.address)
+            for channel in message.opened:
+                if _channel_offset + channel > 64:  # ignore groups
+                    continue
+                await self._update_channel((_channel_offset + channel), {"state": 0})
+            # ignore message.closed: we don't know at what dimlevel they're started
+
+        elif isinstance(message, DimValueStatus):
+            for offset, dim_value in enumerate(message.dim_values):
+                channel = message.channel + offset
+                if channel <= 64:  # channel
+                    await self._update_channel(channel, {"state": dim_value})
+                elif channel <= 80:  # group
+                    group_num = channel - 65
+                    for chan in self.group_members.get(group_num, []):
+                        await self._update_channel(chan, {"state": dim_value})
+                else:  # broadcast
+                    for chan in self._channels.values():
+                        await chan.update({"state": dim_value})
+
+        elif isinstance(
+            message,
+            (
+                SetLedMessage,
+                ClearLedMessage,
+                FastBlinkingLedMessage,
+                SlowBlinkingLedMessage,
+            ),
+        ):
+            pass
+
+        else:
+            return await super().on_message(message)
+
+        self._cache()
+
+    async def _request_channel_name(self) -> None:
+        # Channel names are requested after channel scan
+        # don't do them here (at initialization time)
+        pass
+
+    async def _request_single_channel_name(self, channel_num: int) -> None:
+        msg_type = commandRegistry.get_command(
+            CHANNEL_NAME_REQUEST_COMMAND_CODE, self.get_type()
+        )
+        msg = msg_type(self._address)
+        msg.priority = PRIORITY_LOW
+        msg.channels = channel_num
+        await self._writer(msg)
